from PIL import Image

# control params
sourceFileName = "test.png"
showPreview = True
visualiseDepth = True

source = Image.open(sourceFileName)
preview = Image.new('RGB', source.size,"black")

#--------------------------------------------------------------------------
# write to bool stream
data=[]

# raw for now
def readPixel(x,y):
	return source.getpixel((x,y))[0] > 127
	
def addSquare(x,y,w):
	# more than one pixel?
	#~ print x, y, w, readPixel(x,y)
	if w == 1:
		# nope, just add it
		data.append(readPixel(x,y))
	else:
		# yep, is solid?
		first = readPixel(x,y)
		solid = True
		for dy in range(w):
			for dx in range(w):
				if readPixel(x+dx,y+dy) != first:
					solid = False
					break
			if solid == False:
				break
		
		# write based on whether or not is solid
		if solid:
			data.append(True)
			data.append(first)
		else:
			data.append(False)
			addSquare(x,y,w/2)
			addSquare(x+w/2,y,w/2)
			addSquare(x,y+w/2,w/2)
			addSquare(x+w/2,y+w/2,w/2)

# add the root square
# requires source to be square, POT
addSquare(0,0,source.width)

#--------------------------------------------------------------------------
# print stats
print( str(len(data)) + "/" + str(source.width*source.height) + " bits")
print( str((len(data)+7)/8) + "/" + str((source.width*source.height+7)/8) + " bytes")
print( str(len(data)*100/(source.width*source.height)) + "% of raw file size")

#--------------------------------------------------------------------------
# read from bool stream to preview
readHead=0
def read():
	global readHead
	readHead+=1
	return data[readHead-1]

# palette to use in depth visualisation
# index is the pixel width of the square using this colour
previewPalette={}
previewPalette[1]=(255,255,255)
previewPalette[2]=(255,255,0)
previewPalette[4]=(255,127,0)
previewPalette[8]=(255,0,0)
previewPalette[16]=(127,0,0)
previewPalette[32]=(64,0,0)
previewPalette[64]=(32,0,0)
previewPalette[128]=(0,0,0)

def readSquare(x,y,w):
	# solid?
	if w==1 or read():
		# yes, solid
		# fill with colour
		if read():
			if visualiseDepth == False:
				colour=(255,255,255)
			else:
				colour=(255-previewPalette[w][0], 255-previewPalette[w][1], 255-previewPalette[w][2])
		else:
			if visualiseDepth == False:
				colour=(0,0,0)
			else:
				colour=previewPalette[w]
		for dy in range(w):
			for dx in range(w):
				preview.putpixel((x+dx,y+dy), colour)
	else:
		# no, not solid
		# subdivide
		readSquare(x,y,w/2)
		readSquare(x+w/2,y,w/2)
		readSquare(x,y+w/2,w/2)
		readSquare(x+w/2,y+w/2,w/2)
	
# read the root square
readHead=0
readSquare(0,0,source.width)

#--------------------------------------------------------------------------
# show preview
scale = int( max( min( 1600/preview.width, 800/preview.height ), 1 ) )
preview = preview.resize((source.size[0]*scale, source.size[1]*scale),Image.NEAREST)
preview.show()

